import re

directRules = {
    "naniplay": (re.compile(r"https://(?:www\.)?naniplay(?:\.nanime\.(?:in|biz)|\.com)/file/[^>]+"), "_fembed"),
    "layarkacaxxi": (re.compile(r"https://layarkacaxxi\.icu/f/[^>]+"), "_fembed"),
    "zippyshare": (re.compile(r"https://www\d+\.zippyshare\.com/v/[^/]+/file\.html"), "_zippyshare"),
    "mediafire": (re.compile(r"https://(?:www\.)?mediafire\.com/file/[^>]+"), "_mediafire"),
    "linkpoi": (re.compile(r"https://linkpoi\.me/[^>]+"), "_linkpoi"),
    "uservideo": (re.compile(r"https://(?:www\.)?uservideo\.xyz/file/[^>]+"), "_linkpoi"),
    "ouo": (re.compile(r'https://ouo\.io/[^>]+'), None),
    "zonawibu redirect": (re.compile(r"https://(?:link\.zonawibu\.cc/redirect\.php\?go|player\.zafkiel\.net/blogger\.php\?yuzu)\=[^>]+"), "_redirect")
}

allDirectRules = re.compile(
    r"|".join(v.pattern for v, _ in directRules.values()))



from . import BaseExtractor
from ._rules import directRules, re


class _direct(BaseExtractor):
    """
    Extractor
    """

    def extract_direct_url(self, url):
        end = False
        while not end:
            found_pattern = False
            for title, (pattern, klsname) in directRules.items():
                if klsname and pattern.search(url):
                    if self.logger is not None:
                        self.logger.info(f"Mengekstrak {title}: {url}")
                    func = getattr(self, klsname)
                    try:
                        if not (result := func(url)):
                            break
                        elif not isinstance(result, dict):
                            result = {"": result}
                    except Exception:
                        result = {"": url}
                    url = result[self.choice(result.keys())]
                    found_pattern = True
            if not found_pattern:
                end = True
        return url

    def _redirect(self, url):
        head = self.session.head(url)
        return head.headers.get("Location", url)

    def _linkpoi(self, url):
        raw = self.session.get(url)
        soup = self.soup(raw)

        if (a := soup.find("a", class_="btn-primary")):
            return a["href"]

    def _mediafire(self, url):
        raw = self.session.get(url)
        soup = self.soup(raw)

        if (dl := soup.find(id="downloadButton")):
            return dl["href"]

    def _zippyshare(self, url):
        raw = self.session.get(url)
        res = re.search(
            r'href = "(?P<i>[^"]+)" \+ \((?P<t>[^>]+?)\) \+ "(?P<f>[^"]+)', raw.text)
        if res is not None:
            res = res.groupdict()
            return re.search(r"(^https://www\d+.zippyshare.com)", raw.url).group(1) + \
                res["i"] + str(eval(res["t"])) + res["f"]
        return url

    def _fembed(self, url):
        raw = self.session.get(url)
        api = re.search(r"(/api/source/[^\"']+)", raw.text)
        if api is not None:
            result = {}
            raw = self.session.post(
                "https://layarkacaxxi.icu" + api.group(1)).json()
            for d in raw["data"]:
                f = d["file"]
                direct = self._redirect(f)
                result[f"{d['label']}/{d['type']}"] = direct
            return result



from .thirdparty.exrex import getone as regex_to_string
from shutil import get_terminal_size
from .extractors._rules import allDirectRules
from urllib.parse import urlparse
import argparse
import re


def ArgumentParser(**kwargs):
    extractors = kwargs.pop("extractors")
    version_msg = kwargs.pop("version_msg")

    parser = argparse.ArgumentParser(
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(
            prog, max_help_position=get_terminal_size().lines),
        epilog=version_msg)

    parser.add_argument("query", metavar="query",
                        nargs="*", help="kueri, judul, kata kunci")
    parser.add_argument("-v", "--version", action="store_true",
                        help="show version and exit")
    parser.add_argument("-d", "--debug", action="store_true",
                        help=argparse.SUPPRESS)
    parser.add_argument("-p", metavar="page", dest="page",
                        help=("halaman situs, contoh penggunaan:\n"
                              "  - 1,2,3\n"
                              "  - 1:2, 2:8\n"
                              "  - 1,2,3:8\n"
                              "  - default halaman pertama\n"
                              "    dan seterusnya"), type=str, default="1:")
    parser.add_argument("-i", "--information", dest="info",
                        action="store_true", help="cetak informasi dari item yang dipilih")
    parser.add_argument("--exec", metavar="cmd", dest="_exec",
                        help=("jalankan 'perintah' dengan argument berupa\n"
                              "url yang dipilih.\n\n"
                              "contoh: axel2 {}"))

    valid_e_rules = re.compile(
        r"\[\^.+?\][+*]")
    valid_allDirectRules = valid_e_rules.sub(
        "[A-Za-z0-9-.]{5,7}", allDirectRules.pattern)
    parser.add_argument("-e", "--extract-direct", metavar="url",
                        help=("bypass situs download\n\n"
                              f"contoh url: {regex_to_string(valid_allDirectRules, 3)!r}\n"
                              ))

    parser.add_argument("--list-bypass", action="store_true",
                        help="cetak semua daftar situs download yang dapat\ndi-bypass")

    parser.add_argument("--json-dump", metavar="filename",
                        help="simpan hasil ekstraksi unduhan")
    parser.add_argument("--json", action="store_true",
                        help="cetak hasil ekstraksi unduhan")

    proxies = parser.add_argument_group("Proxies")
    proxies.add_argument("--proxy", metavar="url", help="gunakan HTTP/HTTPS/SOCKS proxy")
    proxies.add_argument("--skip-proxy", action="store_true", help="lewati penggunakan argument --proxy")

    extractor_group = parser.add_argument_group("Daftar Extractor",
                                                description=(
                                                    f"pilih salah satu dari {len(extractors)} situs berikut:"
                                                ))
    extractor_exclusiveGroup = extractor_group.add_mutually_exclusive_group()
    for egn, kls in extractors.items():
        egn = egn.replace("_", "-")
        if hasattr(kls, "host"):
            pa = urlparse(kls.host)
            for index in range(1, len(egn)):
                try:
                    arg = [f"-{egn[:index]}".rstrip("-")]
                    if arg[0] != f"-{egn}":
                        arg.append(f"--{egn}")
                    extractor_exclusiveGroup.add_argument(*arg, action="store_true",
                                                          help=f"site: {pa.scheme}://{pa.netloc} [{kls.tag}]")
                    break
                except argparse.ArgumentError:
                    continue
    return parser
